/** \file
*/

#include "adc.h"
#include "board.h"
#include "buzzer.h"

// ADC field definition for the Mode Register
/* Reminder            TRGEN    => Selection bewteen Software or hardware start of conversion
                       TRGSEL   => Relevant if the previous field set a Hardware Triggering Mode
                 		0x0 << 1 (ADC) Selected TRGSEL = TIAO0
				0x1 << 1 (ADC) Selected TRGSEL = TIAO1
				0x2 << 1 (ADC) Selected TRGSEL = TIAO2
                       LOWRES   => 10-bit result if ths bit is cleared 0
                       SLEEP    => normal mode if ths is cleared
                       PRESCAL  => ADCclock = MCK / [(PRESCAL + 1)*2]
                       STARTUP  => Startup Time = [(STARTUP + 1)*8] / ADCclock
                       SHTIM    => Tracking time = (SHTIM + 1) / ADCclock
 */
#define   TRGEN    (0x1)    ///< Hardware triggering
#define   TRGSEL   (0x0)    ///< Use a Timer output signal (on rising edge) from TIOA0 (for this example)
#define   LOWRES   (0x1)    ///< 8-bit result output
#define   SLEEP    (0x0)    ///< Normal Mode

//#define   PRESCAL  (0x4)    // Max freq value for 10bit; MCK is divided by 10 (ADCClock = MCK / ( (PRESCAL+1) * 2 ))
//#define   PRESCAL  (0x3)    ///< AD CLK = 6MHz (ADCClock = MCK / ( (PRESCAL+1) * 2 ))
#define   PRESCAL  (0x2)	///< AD CLK = 8 MHz

/** \brief This time period must be higher than 20 s and not 20 ms;

	Startup Time = (STARTUP+1) * 8 / ADCClock
*/
#define   STARTUP  (0xf)    

/** \brief Sample & Hold time

	Must be higher than 3 ADC clock cycles but depends on output
	impedance of the analog driver to the ADC input.\n
	Sample & Hold Time = (SHTIM+1) / ADCClock
*/
#define   SHTIM    (0x2)    



/** \brief Data buffers used by ADC
*/
char ADCbuf[ADC_BUF_CNT][4096];
/** \brief Index of current primary ADC buffer for PDC
*/
volatile int iWriteBufId = 0;
/** \brief Index of current ADC buffer for reading
*/
volatile int iReadBufId = -1;


void IRQ_ADC(void)
{
	unsigned int status;
	status = AT91C_BASE_ADC->ADC_SR;

	if(status & AT91C_ADC_ENDRX)
	{
		// The Receive Counter Register has reached 0 since the last write in ADC_RCR or ADC_RNCR.	
		// -> buffer full
		iReadBufId = (iReadBufId+1)%ADC_BUF_CNT;
		iWriteBufId = (iWriteBufId+1)%ADC_BUF_CNT;

		AT91C_BASE_ADC->ADC_RNPR = (unsigned long)(ADCbuf[iWriteBufId]);
		AT91C_BASE_ADC->ADC_RNCR = sizeof(ADCbuf[0])/sizeof(ADCbuf[0][0]);
		//beep();
	}

//	AT91C_BASE_AIC->AIC_ISR;
	/** \note
	The End of Interrupt Command Register is used by the interrupt routine to indicate that the interrupt treatment is complete. 
	Any value can be written because it is only necessary to make a write to this register location to signal the end of interrupt 
	treatment. 
	*/
//	AT91C_BASE_AIC->AIC_EOICR = 0;
}


/** Init A/D converter */
void ADCInit (void)
{
    /* Clear all previous setting and result */
    AT91F_ADC_SoftReset (AT91C_BASE_ADC);
    /* First step: Set the ADC by writing in Mode register */
    AT91F_ADC_CfgModeReg (AT91C_BASE_ADC, (SHTIM << 24) | (STARTUP << 16) | (PRESCAL << 8) |
                          (SLEEP << 5) | (LOWRES <<4) | (TRGSEL << 1) | (TRGEN )) ;
    /* Second Step: Select the active channel */
    AT91F_ADC_EnableChannel (AT91C_BASE_ADC, (1<<CHANNEL1) | (1<<CHANNEL2));	//ustawia ADC_CHER
	
	// Setting interrupt
	AT91C_BASE_AIC->AIC_SVR[AT91C_ID_ADC] = (unsigned long)IRQ_ADC;	
	//  Priority
	AT91C_BASE_AIC->AIC_SMR[AT91C_ID_ADC] = AT91C_AIC_PRIOR_LOWEST;//0x05;
	//  Interrupt Enable Command Register
	AT91C_BASE_AIC->AIC_IECR = (1 << AT91C_ID_ADC);

	
	// Seting PDC (Peripheral DMA Controller) for ADC
	AT91C_BASE_ADC->ADC_PTCR = AT91C_PDC_RXTDIS;
	//  Set receive pointer register
	AT91C_BASE_ADC->ADC_RPR = (unsigned long)ADCbuf;
	//  Set receive counter register (number of transfers), no need to scale with number of channels
	AT91C_BASE_ADC->ADC_RCR = sizeof(ADCbuf[iWriteBufId])/sizeof(ADCbuf[iWriteBufId][0]);	

	iWriteBufId = (iWriteBufId+1)%ADC_BUF_CNT;

	//  Set receive pointer register for next transfer
	AT91C_BASE_ADC->ADC_RNPR = (unsigned int)ADCbuf[iWriteBufId];
	//  Set receive counter register for next transfer
	AT91C_BASE_ADC->ADC_RNCR = sizeof(ADCbuf[iWriteBufId])/sizeof(ADCbuf[iWriteBufId][0]);	

	AT91C_BASE_ADC->ADC_IER = AT91C_ADC_ENDRX;

	//  Enable PDC receiver requests, disable transmitter requests
	//AT91C_BASE_ADC->ADC_PTCR = AT91C_PDC_RXTEN | AT91C_PDC_TXTDIS;
}

void ADCStart (void)
{
	//  Enable PDC receiver requests, disable transmitter requests
	AT91C_BASE_ADC->ADC_PTCR = AT91C_PDC_RXTEN | AT91C_PDC_TXTDIS;
}
